#ifndef FrameCPP_VERSION_6_FrEvent_HH
#define FrameCPP_VERSION_6_FrEvent_HH

#if ! defined(SWIGIMPORTED)
#include <string>
#include <vector>
#endif /* ! defined(SWIGIMPORTED) */

#include "framecpp/FrameCPP.hh"

#include "framecpp/Common/SearchContainer.hh"
#include "framecpp/Common/TOCInfo.hh"

#include "framecpp/Version4/FrTrigData.hh"

#include "framecpp/Version6/FrameSpec.hh"
#include "framecpp/Version6/GPSTime.hh"
#include "framecpp/Version6/FrVect.hh"
#include "framecpp/Version6/FrTable.hh"
#include "framecpp/Version6/STRING.hh"

namespace FrameCPP
{
  namespace Version_6
  {
    //===================================================================
    // FrEventStorage
    //===================================================================
    class FrEventStorage
    {
    public:
      typedef INT_2U				nParam_type;
      typedef std::pair< STRING, REAL_4 >	Param_type;
      typedef std::vector< Param_type >		ParamList_type;

      FrEventStorage( );
      FrEventStorage( const FrEventStorage& Source );
      FrEventStorage( const Previous::FrTrigDataStorage& Source );

      const std::string& GetName() const { return m_data.name; }
      const std::string& GetComment( ) const { return m_data.comment; }
      const std::string& GetInputs( ) const { return m_data.inputs; }
      const GPSTime& GetGTime( ) const { return m_data.GTime; }
      REAL_4 GetTimeBefore( ) const { return m_data.timeBefore; }
      REAL_4 GetTimeAfter( ) const { return m_data.timeAfter; }
      INT_4U GetEventStatus( ) const { return m_data.eventStatus; }
      REAL_4 GetAmplitude( ) const { return m_data.amplitude; }
      REAL_4 GetProbability( ) const { return m_data.probability; }
      const std::string& GetStatistics( ) const { return m_data.statistics; }
      const ParamList_type& GetParam( ) const { return m_data.Params; }
      ParamList_type& GetParam( ) { return m_data.Params; }

    protected:
      struct data_type {
	STRING	name;
	STRING	comment;
	STRING	inputs;
	GPSTime	GTime;
	REAL_4	timeBefore;
	REAL_4	timeAfter;
	INT_4U	eventStatus;
	REAL_4	amplitude;
	REAL_4	probability;
	STRING	statistics;
	ParamList_type	Params;

	data_type( );
	data_type( const data_type& Source );

	cmn_streamsize_type Bytes( ) const;

	bool operator==( const data_type& RHS ) const;

	void operator( )( Common::IStream& Stream );
	void operator( )( Common::OStream& Stream ) const;
      };

      data_type	m_data;
    };

    inline FrEventStorage::data_type::
    data_type( )
    {
    }

    inline FrEventStorage::data_type::
    data_type( const data_type& Source )
      : name( Source.name ),
	comment( Source.comment ),
	inputs( Source.inputs ),
	GTime( Source.GTime ),
	timeBefore( Source.timeBefore ),
	timeAfter( Source.timeAfter ),
	eventStatus( Source.eventStatus ),
	amplitude( Source.amplitude ),
	probability( Source.probability ),
	statistics( Source.statistics ),
	Params( Source.Params )
    {
    }

    inline bool FrEventStorage::data_type::
    operator==( const data_type& RHS ) const
    {
      return ( ( &RHS == this )
	       || ( ( name == RHS.name )
		    && ( comment == RHS.comment )
		    && ( inputs == RHS.inputs )
		    && ( GTime == RHS.GTime )
		    && ( timeBefore == RHS.timeBefore )
		    && ( timeAfter == RHS.timeAfter )
		    && ( eventStatus == RHS.eventStatus )
		    && ( amplitude == RHS.amplitude )
		    && ( probability == RHS.probability )
		    && ( statistics == RHS.statistics )
		    && ( Params == RHS.Params )
		) );
    }


    inline FrEventStorage::
    FrEventStorage( )
    {
    }

    inline FrEventStorage::
    FrEventStorage( const FrEventStorage& Source )
      : m_data( Source.m_data )
    {
    }

    inline FrEventStorage::
    FrEventStorage( const Previous::FrTrigDataStorage& Source )
    {
      m_data.name = Source.GetName( );
      m_data.comment = Source.GetComment( );
      m_data.inputs = Source.GetInputs( );
      m_data.GTime = Source.GetGTime( );
      m_data.timeBefore = Source.GetTimeBefore( );
      m_data.timeAfter = Source.GetTimeAfter( );
      m_data.eventStatus = Source.GetTriggerStatus( );
      m_data.amplitude = Source.GetAmplitude( );
      m_data.probability = Source.GetProbability( );
      m_data.statistics = Source.GetStatistics( );
    }

    //===================================================================
    // FrEventRefsT
    //===================================================================
    template< typename TVect, typename TTable >
    class FrEventRefsT
    {
    public:
      typedef typename Common::SearchContainer< TVect, &TVect::GetName >
      data_type;
      typedef typename data_type::iterator iterator;
      typedef typename data_type::const_iterator const_iterator;

      typedef typename Common::SearchContainer< TTable, &TTable::GetNameSlow >
      table_type;
      typedef typename table_type::iterator table_iterator;
      typedef typename table_type::const_iterator const_table_iterator;

      static const INT_2U MAX_REF = 2;

      FrEventRefsT( )
      {
      }

      FrEventRefsT( const FrEventRefsT& Source )
	: m_refs( Source.m_refs )
      {
      }

      inline const data_type&
      RefData( ) const
      {
	return m_refs.data;
      }

      inline data_type& RefData( )
      {
	return m_refs.data;
      }

      inline const table_type&
      RefTable( ) const
      {
	return m_refs.table;
      }

      inline table_type& RefTable( )
      {
	return m_refs.table;
      }
   
    protected:
      struct ref_type {
	data_type	data;
	table_type	table;

	ref_type( )
	{
	}

	ref_type( const ref_type& Source )
	  : data( Source.data ),
	    table( Source.table )
	{
	}

	inline FrameCPP::cmn_streamsize_type
	Bytes( const Common::StreamBase& Stream ) const
	{
	  return
	    Stream.PtrStructBytes( )	// data
	    + Stream.PtrStructBytes( )	// table
	    ;
	}

	bool
	operator==( const ref_type& RHS ) const       
	{
	  return ( ( this == &RHS )
		   || ( ( data == RHS.data )
			&& ( table == RHS.table )
			) );
	}

	void
	operator( )( Common::IStream& Stream )
	{
	  Stream >> data
		 >> table
	    ;

	}

	void
	operator( )( Common::OStream& Stream ) const
	{
	  Stream << data
		 << table
	    ;

	}

      };

      ref_type	m_refs;
    };

    typedef FrEventRefsT< FrVect, FrTable> FrEventRefs;

    //===================================================================
    // FrEvent
    //===================================================================
    class FrEvent
      : public Common::FrameSpec::Object,
	public FrEventStorage,
	public FrEventRefs,
	public Common::TOCInfo
    {
    public:
      //: Default constructor
      FrEvent( );
      FrEvent( const FrEvent& eventData );
      FrEvent( const std::string& name, const std::string& comment,
	       const std::string& inputs,
	       const GPSTime& time,
	       const REAL_4 timeBefore,
	       const REAL_4 timeAfter,
	       const INT_4U eventStatus,
	       const REAL_4 amplitude,
	       const REAL_4 prob,
	       const std::string& statistics,
	       const ParamList_type& parameters );
      FrEvent( const Previous::FrTrigData& Source,
	       Common::IStream* Stream );

      const std::string& GetNameSlow( ) const;

      static promote_ret_type
      Promote( INT_2U Source,
	       promote_arg_type Obj,
	       Common::IStream* Stream );

      static const char* const StructName( );

      static const Common::Description* StructDescription( );

      //: Number of bytes needed to write this structure
      cmn_streamsize_type Bytes( const Common::StreamBase& Stream) const;

      virtual FrEvent* Create( Common::IStream& Stream ) const;

      virtual const char* const ObjectStructName( ) const;

      virtual void Write( Common::OStream& Stream ) const;

      virtual void TOCQuery( int InfoClass, ... ) const;

      //: Merge with another FrEvent
      FrEvent& Merge( const FrEvent& RHS );

      /// \brief comparison operator
      bool operator==( const FrEvent& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

    protected:
      /// \brief Down grade an object
      virtual demote_ret_type
      demote( INT_2U Target,
	      demote_arg_type Obj,
	      Common::IStream* Stream ) const;

      /// \brief Upgrade an object
      virtual promote_ret_type
      promote( INT_2U Target,
	       promote_arg_type Obj,
	       Common::IStream* Stream ) const;

    private:
      using Common::FrameSpec::Object::Create;

      FrEvent( Common::IStream& Stream );
    };

    inline FrameCPP::cmn_streamsize_type FrEvent::
    Bytes( const Common::StreamBase& Stream ) const
    {
      return
	m_data.Bytes( )
	+ m_refs.Bytes( Stream )
	+ Stream.PtrStructBytes( )	// next
	;
    }

    inline FrEvent::promote_ret_type FrEvent::
    Promote( INT_2U Source,
	     promote_arg_type Obj,
	     Common::IStream* Stream )
    {
      return Object::PromoteObject< Previous::FrTrigData, FrEvent >
	( DATA_FORMAT_VERSION, Source, Obj, Stream );
    }

    inline const char* const FrEvent::
    StructName( )
    {
      static const char* const name = "FrEvent";
      return name;
    }

    inline bool FrEvent::
    operator==( const FrEvent& RHS ) const
    {
      return ( ( this == &RHS )
	       || ( ( m_data == RHS.m_data )
		    && (m_refs == RHS.m_refs )
		    ) );
    }
  } // namespace - Version_6
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_6_FrEvent_HH */
